/*********************************************************************
 * Copyright (c) 2010 Boeing
 *
 * This program and the accompanying materials are made
 * available under the terms of the Eclipse Public License 2.0
 * which is available at https://www.eclipse.org/legal/epl-2.0/
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 * Contributors:
 *     Boeing - initial API and implementation
 **********************************************************************/

package org.eclipse.osee.framework.ui.skynet.render.compare;

import java.util.Collection;
import java.util.Collections;
import java.util.List;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.OperationCanceledException;
import org.eclipse.osee.framework.core.data.BranchToken;
import org.eclipse.osee.framework.core.enums.CoreAttributeTypes;
import org.eclipse.osee.framework.core.enums.PresentationType;
import org.eclipse.osee.framework.core.exception.OperationTimedoutException;
import org.eclipse.osee.framework.core.model.change.CompareData;
import org.eclipse.osee.framework.core.operation.Operations;
import org.eclipse.osee.framework.core.util.RendererOption;
import org.eclipse.osee.framework.jdk.core.result.XResultData;
import org.eclipse.osee.framework.jdk.core.type.OseeArgumentException;
import org.eclipse.osee.framework.jdk.core.type.OseeCoreException;
import org.eclipse.osee.framework.logging.OseeLevel;
import org.eclipse.osee.framework.logging.OseeLog;
import org.eclipse.osee.framework.skynet.core.artifact.Artifact;
import org.eclipse.osee.framework.skynet.core.change.ArtifactDelta;
import org.eclipse.osee.framework.ui.skynet.internal.Activator;
import org.eclipse.osee.framework.ui.skynet.render.FileSystemRenderer;
import org.eclipse.osee.framework.ui.skynet.render.RenderingUtil;
import org.eclipse.osee.framework.ui.skynet.results.XResultDataUI;
import org.eclipse.osee.framework.ui.skynet.util.IVbaDiffGenerator;
import org.eclipse.osee.framework.ui.swt.Displays;

public class WordTemplateCompare extends AbstractWordCompare {

   public WordTemplateCompare(FileSystemRenderer renderer) {
      super(renderer, CoreAttributeTypes.WordTemplateContent, CoreAttributeTypes.PlainTextContent);
   }

   /**
    * Creates a difference report for each artifact between baseArtifact and newerArtifact. Then produces a single
    * report by combining each of the difference reports together for a single report.
    */
   @Override
   public void compareArtifacts(IProgressMonitor monitor, CompareDataCollector collector, PresentationType presentationType, Collection<ArtifactDelta> artifactDeltas, String pathPrefix) {
      if (artifactDeltas.isEmpty()) {
         throw new OseeArgumentException("The artifactDelts must not be empty");
      }

      ArtifactDelta artifactDelta1 = artifactDeltas.iterator().next();
      Artifact artifact = artifactDelta1.getStartArtifact();
      if (artifact == null) {
         artifact = artifactDelta1.getEndArtifact();
      }
      BranchToken branch = artifact.getBranchToken();
      List<Artifact> artifacts = Collections.emptyList();

      IVbaDiffGenerator diffGenerator = createGenerator(artifacts, branch, presentationType);

      String resultPath =
         getDiffPath(artifactDelta1.getStartArtifact(), artifactDelta1.getEndArtifact(), presentationType, pathPrefix);
      String vbsPath =
         RenderingUtil.getRenderPath(getRenderer(), branch, presentationType, null, "compareDocs", ".vbs");

      CompareData data = new CompareData(resultPath, vbsPath);

      addArtifactDeltas(monitor, artifactDeltas, data, presentationType);
      try {
         diffGenerator.generate(monitor, data);
      } catch (OperationTimedoutException ex) {
         if (!skipDialogs) {
            OseeLog.log(Activator.class, OseeLevel.SEVERE_POPUP, String.format(
               "The View Word Change Report Timed-out for Artifact(s) [%s] on Branch [%s]", artifact, branch));
         }
      }

      getRenderer().updateOption(RendererOption.RESULT_PATH_RETURN, resultPath);
      collector.onCompare(data);
   }

   private void addArtifactDeltas(IProgressMonitor monitor, Collection<ArtifactDelta> artifactDeltas, CompareData data, PresentationType presentationType) {
      double workAmount = 0.70 / artifactDeltas.size();

      for (ArtifactDelta artifactDelta : artifactDeltas) {
         if (monitor.isCanceled()) {
            throw new OperationCanceledException();
         }

         try {
            addToCompare(monitor, data, presentationType, artifactDelta);
         } catch (OseeCoreException ex) {
            notifyArtifactDeltaError(ex, artifactDelta);
         } catch (Exception ex) {
            notifyArtifactDeltaError(ex, artifactDelta);
         } finally {
            monitor.worked(Operations.calculateWork(Operations.TASK_WORK_RESOLUTION, workAmount));
         }
      }

      if (monitor.isCanceled()) {
         throw new OperationCanceledException();
      }
   }

   private void notifyArtifactDeltaError(Exception ex, ArtifactDelta artifactDelta) {
      Displays.ensureInDisplayThread(new Runnable() {
         @Override
         public void run() {
            Artifact artifact = artifactDelta.getEndArtifact();
            if (artifact == null) {
               artifact = artifactDelta.getStartArtifact();
            }
            String artString = String.format("%s -- %s", artifact.getIdString(), artifact.getName());
            if (RenderingUtil.arePopupsAllowed()) {
               XResultData rd = new XResultData(false);
               rd.error(String.format("There was a problem when procesing the following artifact for comparison: %s",
                  artString));
               rd.addRaw("Stack Trace");
               for (StackTraceElement element : ex.getStackTrace()) {
                  rd.addRaw("\n");
                  rd.addRaw(element.toString());
               }
               XResultDataUI.report(rd, "Add to Compare Error");
            } else {
               OseeLog.log(Activator.class, OseeLevel.SEVERE_POPUP, ex);
            }
         }
      });
   }
}